/*
 * Copyright LWJGL. All rights reserved.
 * License terms: http://lwjgl.org/license.php
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.opengl;

import org.lwjgl.system.*;

import static org.lwjgl.system.Checks.*;
import static org.lwjgl.system.JNI.*;

/**
 * Native bindings to the <a href="http://www.opengl.org/registry/specs/ARB/ES3_2_compatibility.txt">ARB_ES3_2_compatibility</a> extension.
 * 
 * <p>This extension adds support for features of OpenGL ES 3.2 that are missing from OpenGL 4.5. Enabling these features will ease the process of porting
 * applications from OpenGL ES 3.2 to OpenGL.</p>
 * 
 * <p>In particular this adds the following features:
 * <ul>
 * <li>Bounding box used to optimization tessellation processing (OES_primitive_bounding_box)</li>
 * <li>query for {@link #GL_MULTISAMPLE_LINE_WIDTH_RANGE_ARB MULTISAMPLE_LINE_WIDTH_RANGE_ARB}</li>
 * <li>support for the OpenGL ES 3.20 shading language</li>
 * </ul>
 * For full OpenGL ES 3.2 compatibility the implementation must support {@link KHRBlendEquationAdvanced KHR_blend_equation_advanced} and {@link KHRTextureCompressionASTCLDR KHR_texture_compression_astc_ldr}.
 * Those features are not defined in this extension spec since they are already defined at the KHR level.</p>
 * 
 * <p>Requires {@link GL45 OpenGL 4.5}, {@link ARBES2Compatibility ARB_ES2_compatibility}, {@link ARBES3Compatibility ARB_ES3_compatibility} and {@link ARBES31Compatibility ARB_ES3_1_compatibility}.</p>
 */
public class ARBES32Compatibility {

	/** Accepted by the {@code pname} parameter of GetBooleanv, GetFloatv, GetIntegerv, and GetInteger64v. */
	public static final int
		GL_PRIMITIVE_BOUNDING_BOX_ARB             = 0x92BE,
		GL_MULTISAMPLE_LINE_WIDTH_RANGE_ARB       = 0x9381,
		GL_MULTISAMPLE_LINE_WIDTH_GRANULARITY_ARB = 0x9382;

	/** Function address. */
	@JavadocExclude
	public final long PrimitiveBoundingBoxARB;

	@JavadocExclude
	protected ARBES32Compatibility() {
		throw new UnsupportedOperationException();
	}

	@JavadocExclude
	public ARBES32Compatibility(FunctionProvider provider) {
		PrimitiveBoundingBoxARB = provider.getFunctionAddress("glPrimitiveBoundingBoxARB");
	}

	// --- [ Function Addresses ] ---

	/** Returns the {@link ARBES32Compatibility} instance of the current context. */
	public static ARBES32Compatibility getInstance() {
		return getInstance(GL.getCapabilities());
	}

	/** Returns the {@link ARBES32Compatibility} instance of the specified {@link GLCapabilities}. */
	public static ARBES32Compatibility getInstance(GLCapabilities caps) {
		return checkFunctionality(caps.__ARBES32Compatibility);
	}

	static ARBES32Compatibility create(java.util.Set<String> ext, FunctionProvider provider) {
		if ( !ext.contains("GL_ARB_ES3_2_compatibility") ) return null;

		ARBES32Compatibility funcs = new ARBES32Compatibility(provider);

		boolean supported = checkFunctions(
			funcs.PrimitiveBoundingBoxARB
		);

		return GL.checkExtension("GL_ARB_ES3_2_compatibility", funcs, supported);
	}

	// --- [ glPrimitiveBoundingBoxARB ] ---

	/**
	 * Specifies the primitive bounding box.
	 * 
	 * <p>Implementations may be able to optimize performance if the application provides bounds of primitives that will be generated by the tessellation
	 * primitive generator or the geometry shader prior to executing those stages. If the provided bounds are incorrect and primitives extend beyond them, the
	 * rasterizer may or may not generate fragments for the portions of primitives outside the bounds.</p>
	 *
	 * @param minX the minimum x clip space coordinate
	 * @param minY the minimum y clip space coordinate
	 * @param minZ the minimum z clip space coordinate
	 * @param minW the minimum w clip space coordinate
	 * @param maxX the maximum x clip space coordinate
	 * @param maxY the maximum y clip space coordinate
	 * @param maxZ the maximum z clip space coordinate
	 * @param maxW the maximum w clip space coordinate
	 */
	public static void glPrimitiveBoundingBoxARB(float minX, float minY, float minZ, float minW, float maxX, float maxY, float maxZ, float maxW) {
		long __functionAddress = getInstance().PrimitiveBoundingBoxARB;
		callFFFFFFFFV(__functionAddress, minX, minY, minZ, minW, maxX, maxY, maxZ, maxW);
	}

}